@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -29,19 +29,29 @@ namespace juce
  #define JUCE_COREAUDIOLOG(a)
 #endif
 
-#ifdef __clang__
- #pragma clang diagnostic push
- #pragma clang diagnostic ignored "-Wnonnull" // aovid some spurious 10.11 SDK warnings
-#endif
+JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wnonnull")
+
+constexpr auto juceAudioObjectPropertyElementMain =
+       #if defined (MAC_OS_VERSION_12_0)
+        kAudioObjectPropertyElementMain;
+       #else
+        kAudioObjectPropertyElementMaster;
+       #endif
 
 //==============================================================================
+struct AsyncRestarter
+{
+    virtual ~AsyncRestarter() = default;
+    virtual void restartAsync() = 0;
+};
+
 struct SystemVol
 {
     SystemVol (AudioObjectPropertySelector selector) noexcept
         : outputDeviceID (kAudioObjectUnknown)
     {
         addr.mScope    = kAudioObjectPropertyScopeGlobal;
-        addr.mElement  = kAudioObjectPropertyElementMaster;
+        addr.mElement  = juceAudioObjectPropertyElementMain;
         addr.mSelector = kAudioHardwarePropertyDefaultOutputDevice;
 
         if (AudioObjectHasProperty (kAudioObjectSystemObject, &addr))
@@ -51,7 +61,7 @@ struct SystemVol
 
             if (status == noErr)
             {
-                addr.mElement  = kAudioObjectPropertyElementMaster;
+                addr.mElement  = juceAudioObjectPropertyElementMain;
                 addr.mSelector = selector;
                 addr.mScope    = kAudioDevicePropertyScopeOutput;
 
@@ -124,13 +134,18 @@ private:
     }
 };
 
-#ifdef __clang__
- #pragma clang diagnostic pop
-#endif
+JUCE_END_IGNORE_WARNINGS_GCC_LIKE
+
+constexpr auto juceAudioHardwareServiceDeviceProperty_VirtualMainVolume =
+       #if defined (MAC_OS_VERSION_12_0)
+        kAudioHardwareServiceDeviceProperty_VirtualMainVolume;
+       #else
+        kAudioHardwareServiceDeviceProperty_VirtualMasterVolume;
+       #endif
 
 #define JUCE_SYSTEMAUDIOVOL_IMPLEMENTED 1
-float JUCE_CALLTYPE SystemAudioVolume::getGain()              { return SystemVol (kAudioHardwareServiceDeviceProperty_VirtualMasterVolume).getGain(); }
-bool  JUCE_CALLTYPE SystemAudioVolume::setGain (float gain)   { return SystemVol (kAudioHardwareServiceDeviceProperty_VirtualMasterVolume).setGain (gain); }
+float JUCE_CALLTYPE SystemAudioVolume::getGain()              { return SystemVol (juceAudioHardwareServiceDeviceProperty_VirtualMainVolume).getGain(); }
+bool  JUCE_CALLTYPE SystemAudioVolume::setGain (float gain)   { return SystemVol (juceAudioHardwareServiceDeviceProperty_VirtualMainVolume).setGain (gain); }
 bool  JUCE_CALLTYPE SystemAudioVolume::isMuted()              { return SystemVol (kAudioDevicePropertyMute).isMuted(); }
 bool  JUCE_CALLTYPE SystemAudioVolume::setMuted (bool mute)   { return SystemVol (kAudioDevicePropertyMute).setMuted (mute); }
 
@@ -142,14 +157,15 @@ class CoreAudioIODeviceType;
 class CoreAudioIODevice;
 
 //==============================================================================
-class CoreAudioInternal  : private Timer
+class CoreAudioInternal  : private Timer,
+                           private AsyncUpdater
 {
 public:
     CoreAudioInternal (CoreAudioIODevice& d, AudioDeviceID id, bool input, bool output)
-       : owner (d),
-         deviceID (id),
-         isInputDevice  (input),
-         isOutputDevice (output)
+        : owner (d),
+          deviceID (id),
+          isInputDevice  (input),
+          isOutputDevice (output)
     {
         jassert (deviceID != 0);
 
@@ -169,6 +185,9 @@ public:
 
     ~CoreAudioInternal() override
     {
+        stopTimer();
+        cancelPendingUpdate();
+
         AudioObjectPropertyAddress pa;
         pa.mSelector = kAudioObjectPropertySelectorWildcard;
         pa.mScope = kAudioObjectPropertyScopeWildcard;
@@ -209,7 +228,7 @@ public:
         AudioObjectPropertyAddress pa;
         pa.mSelector = kAudioDevicePropertyStreamConfiguration;
         pa.mScope = input ? kAudioDevicePropertyScopeInput : kAudioDevicePropertyScopeOutput;
-        pa.mElement = kAudioObjectPropertyElementMaster;
+        pa.mElement = juceAudioObjectPropertyElementMain;
 
         if (OK (AudioObjectGetPropertyDataSize (deviceID, &pa, 0, nullptr, &size)))
         {
@@ -264,7 +283,7 @@ public:
 
         AudioObjectPropertyAddress pa;
         pa.mScope = kAudioObjectPropertyScopeWildcard;
-        pa.mElement = kAudioObjectPropertyElementMaster;
+        pa.mElement = juceAudioObjectPropertyElementMain;
         pa.mSelector = kAudioDevicePropertyAvailableNominalSampleRates;
         UInt32 size = 0;
 
@@ -275,7 +294,7 @@ public:
 
             if (OK (AudioObjectGetPropertyData (deviceID, &pa, 0, nullptr, &size, ranges)))
             {
-                for (auto r : { 8000, 11025, 16000, 22050, 32000,
+                for (auto r : { 8000, 11025, 16000, 22050, 24000, 32000,
                                 44100, 48000, 88200, 96000, 176400,
                                 192000, 352800, 384000, 705600, 768000 })
                 {
@@ -305,7 +324,7 @@ public:
 
         AudioObjectPropertyAddress pa;
         pa.mScope = kAudioObjectPropertyScopeWildcard;
-        pa.mElement = kAudioObjectPropertyElementMaster;
+        pa.mElement = juceAudioObjectPropertyElementMain;
         pa.mSelector = kAudioDevicePropertyBufferFrameSizeRange;
         UInt32 size = 0;
 
@@ -346,7 +365,7 @@ public:
         UInt32 latency = 0;
         UInt32 size = sizeof (latency);
         AudioObjectPropertyAddress pa;
-        pa.mElement = kAudioObjectPropertyElementMaster;
+        pa.mElement = juceAudioObjectPropertyElementMain;
         pa.mSelector = kAudioDevicePropertyLatency;
         pa.mScope = scope;
         AudioObjectGetPropertyData (deviceID, &pa, 0, nullptr, &size, &latency);
@@ -362,7 +381,7 @@ public:
     int getBitDepthFromDevice (AudioObjectPropertyScope scope) const
     {
         AudioObjectPropertyAddress pa;
-        pa.mElement = kAudioObjectPropertyElementMaster;
+        pa.mElement = juceAudioObjectPropertyElementMain;
         pa.mSelector = kAudioStreamPropertyPhysicalFormat;
         pa.mScope = scope;
 
@@ -379,7 +398,7 @@ public:
     {
         AudioObjectPropertyAddress pa;
         pa.mScope = kAudioObjectPropertyScopeWildcard;
-        pa.mElement = kAudioObjectPropertyElementMaster;
+        pa.mElement = juceAudioObjectPropertyElementMain;
         pa.mSelector = kAudioDevicePropertyBufferFrameSize;
 
         UInt32 framesPerBuf = (UInt32) bufferSize;
@@ -392,7 +411,7 @@ public:
     {
         AudioObjectPropertyAddress pa;
         pa.mScope = kAudioObjectPropertyScopeWildcard;
-        pa.mElement = kAudioObjectPropertyElementMaster;
+        pa.mElement = juceAudioObjectPropertyElementMain;
         pa.mSelector = kAudioDevicePropertyDeviceIsAlive;
 
         UInt32 isAlive = 0;
@@ -508,7 +527,7 @@ public:
             AudioObjectPropertyAddress pa;
             pa.mSelector = kAudioDevicePropertyDataSourceNameForID;
             pa.mScope = input ? kAudioDevicePropertyScopeInput : kAudioDevicePropertyScopeOutput;
-            pa.mElement = kAudioObjectPropertyElementMaster;
+            pa.mElement = juceAudioObjectPropertyElementMain;
 
             if (OK (AudioObjectGetPropertyData (deviceID, &pa, 0, nullptr, &transSize, &avt)))
                 s.add (buffer);
@@ -526,7 +545,7 @@ public:
         AudioObjectPropertyAddress pa;
         pa.mSelector = kAudioDevicePropertyDataSource;
         pa.mScope = input ? kAudioDevicePropertyScopeInput : kAudioDevicePropertyScopeOutput;
-        pa.mElement = kAudioObjectPropertyElementMaster;
+        pa.mElement = juceAudioObjectPropertyElementMain;
 
         if (deviceID != 0)
         {
@@ -561,7 +580,7 @@ public:
                 AudioObjectPropertyAddress pa;
                 pa.mSelector = kAudioDevicePropertyDataSource;
                 pa.mScope = input ? kAudioDevicePropertyScopeInput : kAudioDevicePropertyScopeOutput;
-                pa.mElement = kAudioObjectPropertyElementMaster;
+                pa.mElement = juceAudioObjectPropertyElementMain;
 
                 OSType typeId = types[index];
 
@@ -575,7 +594,7 @@ public:
         AudioObjectPropertyAddress pa;
         pa.mSelector = kAudioDevicePropertyNominalSampleRate;
         pa.mScope = kAudioObjectPropertyScopeGlobal;
-        pa.mElement = kAudioObjectPropertyElementMaster;
+        pa.mElement = juceAudioObjectPropertyElementMain;
         Float64 sr = 0;
         UInt32 size = (UInt32) sizeof (sr);
         return OK (AudioObjectGetPropertyData (deviceID, &pa, 0, nullptr, &size, &sr)) ? (double) sr : 0.0;
@@ -589,7 +608,7 @@ public:
         AudioObjectPropertyAddress pa;
         pa.mSelector = kAudioDevicePropertyNominalSampleRate;
         pa.mScope = kAudioObjectPropertyScopeGlobal;
-        pa.mElement = kAudioObjectPropertyElementMaster;
+        pa.mElement = juceAudioObjectPropertyElementMain;
         Float64 sr = newSampleRate;
         return OK (AudioObjectSetPropertyData (deviceID, &pa, 0, nullptr, sizeof (sr), &sr));
     }
@@ -631,7 +650,7 @@ public:
             AudioObjectPropertyAddress pa;
             pa.mSelector = kAudioDevicePropertyBufferFrameSize;
             pa.mScope = kAudioObjectPropertyScopeGlobal;
-            pa.mElement = kAudioObjectPropertyElementMaster;
+            pa.mElement = juceAudioObjectPropertyElementMain;
             UInt32 framesPerBuf = (UInt32) bufferSizeSamples;
 
             if (! OK (AudioObjectSetPropertyData (deviceID, &pa, 0, nullptr, sizeof (framesPerBuf), &framesPerBuf)))
@@ -660,17 +679,23 @@ public:
         return error;
     }
 
-    bool start()
+    bool start (AudioIODeviceCallback* callbackToNotify)
     {
+        const ScopedLock sl (callbackLock);
+
+        if (callback == nullptr && callbackToNotify != nullptr)
+        {
+            callback = callbackToNotify;
+            callback->audioDeviceAboutToStart (&owner);
+        }
+
         if (! started)
         {
-            callback = nullptr;
-
             if (deviceID != 0)
             {
                 if (OK (AudioDeviceCreateIOProcID (deviceID, audioIOProc, this, &audioProcID)))
                 {
-                    if (OK (AudioDeviceStart (deviceID, audioIOProc)))
+                    if (OK (AudioDeviceStart (deviceID, audioProcID)))
                     {
                         started = true;
                     }
@@ -683,53 +708,38 @@ public:
             }
         }
 
+        playing = started && callback != nullptr;
+
         return started;
     }
 
-    void setCallback (AudioIODeviceCallback* cb)
+    AudioIODeviceCallback* stop (bool leaveInterruptRunning)
     {
         const ScopedLock sl (callbackLock);
-        callback = cb;
-    }
 
-    void stop (bool leaveInterruptRunning)
-    {
-        {
-            const ScopedLock sl (callbackLock);
-            callback = nullptr;
-        }
+        auto result = std::exchange (callback, nullptr);
 
         if (started && (deviceID != 0) && ! leaveInterruptRunning)
         {
-            OK (AudioDeviceStop (deviceID, audioIOProc));
-            OK (AudioDeviceDestroyIOProcID (deviceID, audioProcID));
-            audioProcID = {};
+            audioDeviceStopPending = true;
 
-            started = false;
-
-            { const ScopedLock sl (callbackLock); }
-
-            // wait until it's definitely stopped calling back..
+            // wait until AudioDeviceStop() has been called on the IO thread
             for (int i = 40; --i >= 0;)
             {
-                Thread::sleep (50);
-
-                UInt32 running = 0;
-                UInt32 size = sizeof (running);
-
-                AudioObjectPropertyAddress pa;
-                pa.mSelector = kAudioDevicePropertyDeviceIsRunning;
-                pa.mScope = kAudioObjectPropertyScopeWildcard;
-                pa.mElement = kAudioObjectPropertyElementMaster;
-
-                OK (AudioObjectGetPropertyData (deviceID, &pa, 0, nullptr, &size, &running));
-
-                if (running == 0)
+                if (audioDeviceStopPending == false)
                     break;
+
+                const ScopedUnlock ul (callbackLock);
+                Thread::sleep (50);
             }
 
-            const ScopedLock sl (callbackLock);
+            OK (AudioDeviceDestroyIOProcID (deviceID, audioProcID));
+            audioProcID = {};
+            started = false;
+            playing = false;
         }
+
+        return result;
     }
 
     double getSampleRate() const  { return sampleRate; }
@@ -740,6 +750,14 @@ public:
     {
         const ScopedLock sl (callbackLock);
 
+        if (audioDeviceStopPending)
+        {
+            if (OK (AudioDeviceStop (deviceID, audioProcID)))
+                audioDeviceStopPending = false;
+
+            return;
+        }
+
         if (callback != nullptr)
         {
             for (int i = numInputChans; --i >= 0;)
@@ -790,13 +808,52 @@ public:
         }
     }
 
-    // called by callbacks
+    // called by callbacks (possibly off the main thread)
     void deviceDetailsChanged()
     {
         if (callbacksAllowed.get() == 1)
             startTimer (100);
     }
 
+    // called by callbacks (possibly off the main thread)
+    void deviceRequestedRestart()
+    {
+        owner.restart();
+        triggerAsyncUpdate();
+    }
+
+    bool isPlaying() const { return playing.load(); }
+
+    //==============================================================================
+    CoreAudioIODevice& owner;
+    int inputLatency  = 0;
+    int outputLatency = 0;
+    int bitDepth = 32;
+    int xruns = 0;
+    BigInteger activeInputChans, activeOutputChans;
+    StringArray inChanNames, outChanNames;
+    Array<double> sampleRates;
+    Array<int> bufferSizes;
+    AudioDeviceIOProcID audioProcID = {};
+
+private:
+    AudioIODeviceCallback* callback = nullptr;
+    CriticalSection callbackLock;
+    AudioDeviceID deviceID;
+    bool started = false, audioDeviceStopPending = false;
+    std::atomic<bool> playing { false };
+    double sampleRate = 0;
+    int bufferSize = 512;
+    HeapBlock<float> audioBuffer;
+    int numInputChans  = 0;
+    int numOutputChans = 0;
+    Atomic<int> callbacksAllowed { 1 };
+    const bool isInputDevice, isOutputDevice;
+
+    Array<CallbackDetailsForChannel> inputChannelInfo, outputChannelInfo;
+    HeapBlock<float*> tempInputBuffers, tempOutputBuffers;
+
+    //==============================================================================
     void timerCallback() override
     {
         JUCE_COREAUDIOLOG ("Device changed");
@@ -811,35 +868,12 @@ public:
             owner.restart();
     }
 
-    //==============================================================================
-    CoreAudioIODevice& owner;
-    int inputLatency  = 0;
-    int outputLatency = 0;
-    int bitDepth = 32;
-    int xruns = 0;
-    BigInteger activeInputChans, activeOutputChans;
-    StringArray inChanNames, outChanNames;
-    Array<double> sampleRates;
-    Array<int> bufferSizes;
-    AudioIODeviceCallback* callback = nullptr;
-    AudioDeviceIOProcID audioProcID = {};
+    void handleAsyncUpdate() override
+    {
+        if (owner.deviceType != nullptr)
+            owner.deviceType->audioDeviceListChanged();
+    }
 
-private:
-    CriticalSection callbackLock;
-    AudioDeviceID deviceID;
-    bool started = false;
-    double sampleRate = 0;
-    int bufferSize = 512;
-    HeapBlock<float> audioBuffer;
-    int numInputChans  = 0;
-    int numOutputChans = 0;
-    Atomic<int> callbacksAllowed { 1 };
-    const bool isInputDevice, isOutputDevice;
-
-    Array<CallbackDetailsForChannel> inputChannelInfo, outputChannelInfo;
-    HeapBlock<float*> tempInputBuffers, tempOutputBuffers;
-
-    //==============================================================================
     static OSStatus audioIOProc (AudioDeviceID /*inDevice*/,
                                  const AudioTimeStamp* /*inNow*/,
                                  const AudioBufferList* inInputData,
@@ -874,10 +908,7 @@ private:
 
             case kAudioDevicePropertyDeviceHasChanged:
             case kAudioObjectPropertyOwnedObjects:
-                intern->owner.restart();
-
-                if (intern->owner.deviceType != nullptr)
-                    intern->owner.deviceType->triggerAsyncAudioDeviceListChange();
+                intern->deviceRequestedRestart();
                 break;
 
             case kAudioDevicePropertyBufferSizeRange:
@@ -898,7 +929,7 @@ private:
         AudioObjectPropertyAddress pa;
         pa.mSelector = kAudioDevicePropertyDataSources;
         pa.mScope = kAudioObjectPropertyScopeWildcard;
-        pa.mElement = kAudioObjectPropertyElementMaster;
+        pa.mElement = juceAudioObjectPropertyElementMain;
         UInt32 size = 0;
 
         if (deviceID != 0
@@ -945,20 +976,18 @@ public:
           inputIndex (inputIndex_),
           outputIndex (outputIndex_)
     {
-        CoreAudioInternal* device = nullptr;
-
-        if (outputDeviceId == 0 || outputDeviceId == inputDeviceId)
+        internal = [this, &inputDeviceId, &outputDeviceId]
         {
-            jassert (inputDeviceId != 0);
-            device = new CoreAudioInternal (*this, inputDeviceId, true, outputDeviceId != 0);
-        }
-        else
-        {
-            device = new CoreAudioInternal (*this, outputDeviceId, false, true);
-        }
+            if (outputDeviceId == 0 || outputDeviceId == inputDeviceId)
+            {
+                jassert (inputDeviceId != 0);
+                return std::make_unique<CoreAudioInternal> (*this, inputDeviceId, true, outputDeviceId != 0);
+            }
 
-        jassert (device != nullptr);
-        internal.reset (device);
+            return std::make_unique<CoreAudioInternal> (*this, outputDeviceId, false, true);
+        }();
+
+        jassert (internal != nullptr);
 
         AudioObjectPropertyAddress pa;
         pa.mSelector = kAudioObjectPropertySelectorWildcard;
@@ -1054,49 +1083,35 @@ public:
 
     void start (AudioIODeviceCallback* callback) override
     {
-        if (! isStarted)
-        {
-            if (callback != nullptr)
-                callback->audioDeviceAboutToStart (this);
-
-            isStarted = internal->start();
-
-            if (isStarted)
-            {
-                internal->setCallback (callback);
-                previousCallback = callback;
-            }
-        }
+        if (internal->start (callback))
+            previousCallback = callback;
     }
 
     void stop() override
     {
         restartDevice = false;
-
-        if (isStarted)
-        {
-            auto lastCallback = internal->callback;
-
-            isStarted = false;
-            internal->stop (true);
-
-            if (lastCallback != nullptr)
-                lastCallback->audioDeviceStopped();
-        }
+        stopAndGetLastCallback();
     }
 
-    void stopInternal()
+    AudioIODeviceCallback* stopAndGetLastCallback() const
+    {
+        auto* lastCallback = internal->stop (true);
+
+        if (lastCallback != nullptr)
+            lastCallback->audioDeviceStopped();
+
+        return lastCallback;
+    }
+
+    AudioIODeviceCallback* stopInternal()
     {
-        stop();
         restartDevice = true;
+        return stopAndGetLastCallback();
     }
 
     bool isPlaying() override
     {
-        if (internal->callback == nullptr)
-            isStarted = false;
-
-        return isStarted;
+        return internal->isPlaying();
     }
 
     String getLastError() override
@@ -1110,28 +1125,21 @@ public:
             deviceType->audioDeviceListChanged();
     }
 
+    // called by callbacks (possibly off the main thread)
     void restart()
     {
-        if (deviceWrapperRestartCallback != nullptr)
+        if (restarter != nullptr)
         {
-            deviceWrapperRestartCallback();
+            restarter->restartAsync();
+            return;
         }
-        else
+
         {
-            {
-                const ScopedLock sl (closeLock);
-
-                if (isStarted)
-                {
-                    if (internal->callback != nullptr)
-                        previousCallback = internal->callback;
-
-                    stopInternal();
-                }
-            }
-
-            startTimer (100);
+            const ScopedLock sl (closeLock);
+            previousCallback = stopInternal();
         }
+
+        startTimer (100);
     }
 
     bool setCurrentSampleRate (double newSampleRate)
@@ -1139,9 +1147,9 @@ public:
         return internal->setNominalSampleRate (newSampleRate);
     }
 
-    void setDeviceWrapperRestartCallback (const std::function<void()>& cb)
+    void setAsyncRestarter (AsyncRestarter* restarterIn)
     {
-        deviceWrapperRestartCallback = cb;
+        restarter = restarterIn;
     }
 
     bool shouldRestartDevice() const noexcept    { return restartDevice; }
@@ -1151,10 +1159,10 @@ public:
 
 private:
     std::unique_ptr<CoreAudioInternal> internal;
-    bool isOpen_ = false, isStarted = false, restartDevice = true;
+    bool isOpen_ = false, restartDevice = true;
     String lastError;
     AudioIODeviceCallback* previousCallback = nullptr;
-    std::function<void()> deviceWrapperRestartCallback = nullptr;
+    AsyncRestarter* restarter = nullptr;
     BigInteger inputChannelsRequested, outputChannelsRequested;
     CriticalSection closeLock;
 
@@ -1193,6 +1201,7 @@ private:
 
 //==============================================================================
 class AudioIODeviceCombiner    : public AudioIODevice,
+                                 private AsyncRestarter,
                                  private Thread,
                                  private Timer
 {
@@ -1210,18 +1219,20 @@ public:
         devices.clear();
     }
 
-    void addDevice (CoreAudioIODevice* device, bool useInputs, bool useOutputs)
+    void addDevice (std::unique_ptr<CoreAudioIODevice> device, bool useInputs, bool useOutputs)
     {
         jassert (device != nullptr);
         jassert (! isOpen());
         jassert (! device->isOpen());
-        devices.add (new DeviceWrapper (*this, device, useInputs, useOutputs));
+        auto* devicePtr = device.get();
+
+        devices.add (std::make_unique<DeviceWrapper> (*this, std::move (device), useInputs, useOutputs));
 
         if (currentSampleRate == 0)
-            currentSampleRate = device->getCurrentSampleRate();
+            currentSampleRate = devicePtr->getCurrentSampleRate();
 
         if (currentBufferSize == 0)
-            currentBufferSize = device->getCurrentBufferSizeSamples();
+            currentBufferSize = devicePtr->getCurrentBufferSizeSamples();
     }
 
     Array<AudioIODevice*> getDevices() const
@@ -1449,7 +1460,7 @@ public:
         start (cb);
     }
 
-    void restartAsync()
+    void restartAsync() override
     {
         {
             const ScopedLock sl (closeLock);
@@ -1526,7 +1537,13 @@ public:
 
     void start (AudioIODeviceCallback* newCallback) override
     {
-        if (callback != newCallback)
+        const auto shouldStart = [&]
+        {
+            const ScopedLock sl (callbackLock);
+            return callback != newCallback;
+        }();
+
+        if (shouldStart)
         {
             stop();
             fifos.clear();
@@ -1538,8 +1555,7 @@ public:
                 newCallback->audioDeviceAboutToStart (this);
 
             const ScopedLock sl (callbackLock);
-            callback = newCallback;
-            previousCallback = callback;
+            previousCallback = callback = newCallback;
         }
     }
 
@@ -1668,11 +1684,16 @@ private:
     void readInput (AudioBuffer<float>& buffer, const int numSamples, const int blockSizeMs)
     {
         for (auto* d : devices)
-            d->done = (d->numInputChans == 0);
+            d->done = (d->numInputChans == 0 || d->isWaitingForInput);
 
-        for (int tries = 5;;)
+        float totalWaitTimeMs = blockSizeMs * 5.0f;
+        constexpr int numReadAttempts = 6;
+        auto sumPower2s = [] (int maxPower) { return (1 << (maxPower + 1)) - 1; };
+        float waitTime = totalWaitTimeMs / (float) sumPower2s (numReadAttempts - 2);
+
+        for (int numReadAttemptsRemaining = numReadAttempts;;)
         {
-            bool anyRemaining = false;
+            bool anySamplesRemaining = false;
 
             for (auto* d : devices)
             {
@@ -1684,17 +1705,20 @@ private:
                         d->done = true;
                     }
                     else
-                        anyRemaining = true;
+                    {
+                        anySamplesRemaining = true;
+                    }
                 }
             }
 
-            if (! anyRemaining)
+            if (! anySamplesRemaining)
                 return;
 
-            if (--tries == 0)
+            if (--numReadAttemptsRemaining == 0)
                 break;
 
-            wait (blockSizeMs);
+            wait (jmax (1, roundToInt (waitTime)));
+            waitTime *= 2.0f;
         }
 
         for (auto* d : devices)
@@ -1722,7 +1746,9 @@ private:
                         d->done = true;
                     }
                     else
+                    {
                         anyRemaining = true;
+                    }
                 }
             }
 
@@ -1786,11 +1812,11 @@ private:
     //==============================================================================
     struct DeviceWrapper  : private AudioIODeviceCallback
     {
-        DeviceWrapper (AudioIODeviceCombiner& cd, CoreAudioIODevice* d, bool useIns, bool useOuts)
-            : owner (cd), device (d),
+        DeviceWrapper (AudioIODeviceCombiner& cd, std::unique_ptr<CoreAudioIODevice> d, bool useIns, bool useOuts)
+            : owner (cd), device (std::move (d)),
               useInputs (useIns), useOutputs (useOuts)
         {
-            d->setDeviceWrapperRestartCallback ([this] { owner.restartAsync(); });
+            device->setAsyncRestarter (&owner);
         }
 
         ~DeviceWrapper() override
@@ -1813,6 +1839,8 @@ private:
             numInputChans  = useInputs  ? device->getActiveInputChannels().countNumberOfSetBits()  : 0;
             numOutputChans = useOutputs ? device->getActiveOutputChannels().countNumberOfSetBits() : 0;
 
+            isWaitingForInput = numInputChans > 0;
+
             inputIndex = channelIndex;
             outputIndex = channelIndex + numInputChans;
 
@@ -1897,6 +1925,8 @@ private:
         {
             if (numInputChannels > 0)
             {
+                isWaitingForInput = false;
+
                 int start1, size1, start2, size2;
                 inputFifo.prepareToWrite (numSamples, start1, size1, start2, size2);
 
@@ -1978,6 +2008,7 @@ private:
         std::unique_ptr<CoreAudioIODevice> device;
         int inputIndex = 0, numInputChans = 0, outputIndex = 0, numOutputChans = 0;
         bool useInputs = false, useOutputs = false;
+        std::atomic<bool> isWaitingForInput { false };
         AbstractFifo inputFifo { 32 }, outputFifo { 32 };
         bool done = false;
 
@@ -2007,6 +2038,8 @@ public:
 
     ~CoreAudioIODeviceType() override
     {
+        cancelPendingUpdate();
+
         AudioObjectPropertyAddress pa;
         pa.mSelector = kAudioHardwarePropertyDevices;
         pa.mScope = kAudioObjectPropertyScopeWildcard;
@@ -2030,7 +2063,7 @@ public:
         AudioObjectPropertyAddress pa;
         pa.mSelector = kAudioHardwarePropertyDevices;
         pa.mScope = kAudioObjectPropertyScopeWildcard;
-        pa.mElement = kAudioObjectPropertyElementMaster;
+        pa.mElement = juceAudioObjectPropertyElementMain;
 
         if (AudioObjectGetPropertyDataSize (kAudioObjectSystemObject, &pa, 0, nullptr, &size) == noErr)
         {
@@ -2095,7 +2128,7 @@ public:
         pa.mSelector = forInput ? kAudioHardwarePropertyDefaultInputDevice
                                 : kAudioHardwarePropertyDefaultOutputDevice;
         pa.mScope    = kAudioObjectPropertyScopeWildcard;
-        pa.mElement  = kAudioObjectPropertyElementMaster;
+        pa.mElement  = juceAudioObjectPropertyElementMain;
 
         if (AudioObjectGetPropertyData (kAudioObjectSystemObject, &pa, 0, nullptr, &size, &deviceID) == noErr)
         {
@@ -2158,22 +2191,20 @@ public:
                                                        : outputDeviceName;
 
         if (inputDeviceID == outputDeviceID)
-            return new CoreAudioIODevice (this, combinedName, inputDeviceID, inputIndex, outputDeviceID, outputIndex);
+            return std::make_unique<CoreAudioIODevice> (this, combinedName, inputDeviceID, inputIndex, outputDeviceID, outputIndex).release();
 
-        std::unique_ptr<CoreAudioIODevice> in, out;
+        auto in = inputDeviceID != 0 ? std::make_unique<CoreAudioIODevice> (this, inputDeviceName, inputDeviceID, inputIndex, 0, -1)
+                                     : nullptr;
 
-        if (inputDeviceID != 0)
-            in.reset (new CoreAudioIODevice (this, inputDeviceName, inputDeviceID, inputIndex, 0, -1));
+        auto out = outputDeviceID != 0 ? std::make_unique<CoreAudioIODevice> (this, outputDeviceName, 0, -1, outputDeviceID, outputIndex)
+                                       : nullptr;
 
-        if (outputDeviceID != 0)
-            out.reset (new CoreAudioIODevice (this, outputDeviceName, 0, -1, outputDeviceID, outputIndex));
-
-        if (in == nullptr)   return out.release();
+        if (in  == nullptr)  return out.release();
         if (out == nullptr)  return in.release();
 
-        std::unique_ptr<AudioIODeviceCombiner> combo (new AudioIODeviceCombiner (combinedName, this));
-        combo->addDevice (in.release(),  true, false);
-        combo->addDevice (out.release(), false, true);
+        auto combo = std::make_unique<AudioIODeviceCombiner> (combinedName, this);
+        combo->addDevice (std::move (in),  true, false);
+        combo->addDevice (std::move (out), false, true);
         return combo.release();
     }
 
@@ -2183,11 +2214,6 @@ public:
         callDeviceChangeListeners();
     }
 
-    void triggerAsyncAudioDeviceListChange()
-    {
-        triggerAsyncUpdate();
-    }
-
     //==============================================================================
 private:
     StringArray inputDeviceNames, outputDeviceNames;
@@ -2195,6 +2221,11 @@ private:
 
     bool hasScanned = false;
 
+    void handleAsyncUpdate() override
+    {
+        audioDeviceListChanged();
+    }
+
     static int getNumChannels (AudioDeviceID deviceID, bool input)
     {
         int total = 0;
@@ -2203,7 +2234,7 @@ private:
         AudioObjectPropertyAddress pa;
         pa.mSelector = kAudioDevicePropertyStreamConfiguration;
         pa.mScope = input ? kAudioDevicePropertyScopeInput : kAudioDevicePropertyScopeOutput;
-        pa.mElement = kAudioObjectPropertyElementMaster;
+        pa.mElement = juceAudioObjectPropertyElementMain;
 
         if (AudioObjectGetPropertyDataSize (deviceID, &pa, 0, nullptr, &size) == noErr)
         {
@@ -2224,27 +2255,16 @@ private:
 
     static OSStatus hardwareListenerProc (AudioDeviceID, UInt32, const AudioObjectPropertyAddress*, void* clientData)
     {
-        static_cast<CoreAudioIODeviceType*> (clientData)->triggerAsyncAudioDeviceListChange();
+        static_cast<CoreAudioIODeviceType*> (clientData)->triggerAsyncUpdate();
         return noErr;
     }
 
-    void handleAsyncUpdate() override
-    {
-        audioDeviceListChanged();
-    }
-
     JUCE_DECLARE_WEAK_REFERENCEABLE (CoreAudioIODeviceType)
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (CoreAudioIODeviceType)
 };
 
 };
 
-//==============================================================================
-AudioIODeviceType* AudioIODeviceType::createAudioIODeviceType_CoreAudio()
-{
-    return new CoreAudioClasses::CoreAudioIODeviceType();
-}
-
 #undef JUCE_COREAUDIOLOG
 
 } // namespace juce
