@@ -28,8 +28,8 @@ namespace juce
 
 struct InternalWebViewType
 {
-    InternalWebViewType() {}
-    virtual ~InternalWebViewType() {}
+    InternalWebViewType() = default;
+    virtual ~InternalWebViewType() = default;
 
     virtual void createBrowser() = 0;
     virtual bool hasBrowserBeenCreated() = 0;
@@ -47,20 +47,6 @@ struct InternalWebViewType
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (InternalWebViewType)
 };
 
-#if JUCE_MINGW
- JUCE_DECLARE_UUID_GETTER (IOleClientSite,           "00000118-0000-0000-c000-000000000046")
- JUCE_DECLARE_UUID_GETTER (IDispatch,                "00020400-0000-0000-c000-000000000046")
-
- #ifndef WebBrowser
-  class WebBrowser;
- #endif
-#endif
-
-JUCE_DECLARE_UUID_GETTER (DWebBrowserEvents2,        "34A715A0-6587-11D0-924A-0020AFC7AC4D")
-JUCE_DECLARE_UUID_GETTER (IConnectionPointContainer, "B196B284-BAB4-101A-B69C-00AA00341D07")
-JUCE_DECLARE_UUID_GETTER (IWebBrowser2,              "D30C1661-CDAF-11D0-8A3E-00C04FC9E26E")
-JUCE_DECLARE_UUID_GETTER (WebBrowser,                "8856F961-340A-11D0-A96B-00C04FD705A2")
-
 //==============================================================================
 class Win32WebView   : public InternalWebViewType,
                        public ActiveXControlComponent
@@ -342,11 +328,12 @@ class WebView2  : public InternalWebViewType,
                   public ComponentMovementWatcher
 {
 public:
-    WebView2 (WebBrowserComponent& o, const File& dllLocation, const File& userDataFolder)
+    WebView2 (WebBrowserComponent& o, const WebView2Preferences& prefs)
          : ComponentMovementWatcher (&o),
-           owner (o)
+           owner (o),
+           preferences (prefs)
     {
-        if (! createWebViewEnvironment (dllLocation, userDataFolder))
+        if (! createWebViewEnvironment())
             throw std::runtime_error ("Failed to create the CoreWebView2Environemnt");
 
         owner.addAndMakeVisible (this);
@@ -604,13 +591,38 @@ private:
         }
     }
 
-    bool createWebViewEnvironment (const File& dllLocation, const File& userDataFolder)
+    void setWebViewPreferences()
+    {
+        ComSmartPtr<ICoreWebView2Controller2> controller2;
+        webViewController->QueryInterface (controller2.resetAndGetPointerAddress());
+
+        if (controller2 != nullptr)
+        {
+            const auto bgColour = preferences.getBackgroundColour();
+
+            controller2->put_DefaultBackgroundColor ({ (BYTE) bgColour.getAlpha(),
+                                                       (BYTE) bgColour.getRed(),
+                                                       (BYTE) bgColour.getGreen(),
+                                                       (BYTE) bgColour.getBlue() });
+        }
+
+        ComSmartPtr<ICoreWebView2Settings> settings;
+        webView->get_Settings (settings.resetAndGetPointerAddress());
+
+        if (settings != nullptr)
+        {
+            settings->put_IsStatusBarEnabled (! preferences.getIsStatusBarDisabled());
+            settings->put_IsBuiltInErrorPageEnabled (! preferences.getIsBuiltInErrorPageDisabled());
+        }
+    }
+
+    bool createWebViewEnvironment()
     {
         using CreateWebViewEnvironmentWithOptionsFunc = HRESULT (*) (PCWSTR, PCWSTR,
                                                                      ICoreWebView2EnvironmentOptions*,
                                                                      ICoreWebView2CreateCoreWebView2EnvironmentCompletedHandler*);
 
-        auto dllPath = dllLocation.getFullPathName();
+        auto dllPath = preferences.getDLLLocation().getFullPathName();
 
         if (dllPath.isEmpty())
             dllPath = "WebView2Loader.dll";
@@ -630,9 +642,10 @@ private:
         }
 
         auto options = Microsoft::WRL::Make<CoreWebView2EnvironmentOptions>();
+        const auto userDataFolder = preferences.getUserDataFolder().getFullPathName();
 
         auto hr = createWebViewEnvironmentWithOptions (nullptr,
-                                                       userDataFolder != File() ? userDataFolder.getFullPathName().toWideCharPointer() : nullptr,
+                                                       userDataFolder.isNotEmpty() ? userDataFolder.toWideCharPointer() : nullptr,
                                                        options.Get(),
             Callback<ICoreWebView2CreateCoreWebView2EnvironmentCompletedHandler>(
                 [weakThis = WeakReference<WebView2> { this }] (HRESULT, ICoreWebView2Environment* env) -> HRESULT
@@ -667,11 +680,15 @@ private:
                                 weakThis->webViewController = controller;
                                 controller->get_CoreWebView2 (weakThis->webView.resetAndGetPointerAddress());
 
-                                weakThis->addEventHandlers();
-                                weakThis->componentMovedOrResized (true, true);
+                                if (weakThis->webView != nullptr)
+                                {
+                                    weakThis->addEventHandlers();
+                                    weakThis->setWebViewPreferences();
+                                    weakThis->componentMovedOrResized (true, true);
 
-                                if (weakThis->webView != nullptr && weakThis->urlRequest.url.isNotEmpty())
-                                    weakThis->webView->Navigate (weakThis->urlRequest.url.toWideCharPointer());
+                                    if (weakThis->urlRequest.url.isNotEmpty())
+                                        weakThis->webView->Navigate (weakThis->urlRequest.url.toWideCharPointer());
+                                }
                             }
                         }
 
@@ -715,6 +732,7 @@ private:
 
     //==============================================================================
     WebBrowserComponent& owner;
+    WebView2Preferences preferences;
 
     HMODULE webView2LoaderHandle = nullptr;
 
@@ -734,6 +752,7 @@ private:
         StringArray headers;
         MemoryBlock postData;
     };
+
     URLRequest urlRequest;
 
     bool isCreating = false;
@@ -749,20 +768,22 @@ private:
 class WebBrowserComponent::Pimpl
 {
 public:
-    Pimpl (WebBrowserComponent& owner, const File& dllLocation, const File& userDataFolder, bool useWebView2)
+    Pimpl (WebBrowserComponent& owner,
+           const WebView2Preferences& preferences,
+           bool useWebView2)
     {
         if (useWebView2)
         {
            #if JUCE_USE_WIN_WEBVIEW2
             try
             {
-                internal.reset (new WebView2 (owner, dllLocation, userDataFolder));
+                internal.reset (new WebView2 (owner, preferences));
             }
-            catch (std::runtime_error&) {}
+            catch (const std::runtime_error&) {}
            #endif
         }
 
-        ignoreUnused (dllLocation, userDataFolder);
+        ignoreUnused (preferences);
 
         if (internal == nullptr)
             internal.reset (new Win32WebView (owner));
@@ -779,17 +800,14 @@ private:
 
 //==============================================================================
 WebBrowserComponent::WebBrowserComponent (bool unloadWhenHidden)
-    : browser (new Pimpl (*this, {}, {}, false)),
+    : browser (new Pimpl (*this, {}, false)),
       unloadPageWhenHidden (unloadWhenHidden)
 {
     setOpaque (true);
 }
 
-WebBrowserComponent::WebBrowserComponent (bool unloadWhenHidden,
-                                          const File& dllLocation,
-                                          const File& userDataFolder)
-    : browser (new Pimpl (*this, dllLocation, userDataFolder, true)),
-      unloadPageWhenHidden (unloadWhenHidden)
+WebBrowserComponent::WebBrowserComponent (ConstructWithoutPimpl args)
+    : unloadPageWhenHidden (args.unloadWhenHidden)
 {
     setOpaque (true);
 }
@@ -798,6 +816,13 @@ WebBrowserComponent::~WebBrowserComponent()
 {
 }
 
+WindowsWebView2WebBrowserComponent::WindowsWebView2WebBrowserComponent (bool unloadWhenHidden,
+                                                                        const WebView2Preferences& preferences)
+    : WebBrowserComponent (ConstructWithoutPimpl { unloadWhenHidden })
+{
+    browser = std::make_unique<Pimpl> (*this, preferences, true);
+}
+
 //==============================================================================
 void WebBrowserComponent::goToURL (const String& url,
                                    const StringArray* headers,
